import { css, html, LitElement, property, svg } from "lit-element";

import "./hat-graph-node";

const BRANCH_HEIGHT = 30;
const BRANCH_CURVATURE = 25;
const VERTICAL_SPACING = 10;
const NODE_SIZE = 24;

export class HatGraph extends LitElement {
  @property() _num_items = 0;

  @property({ reflect: true }) branching?;
  config?;

  updateNode(config) {
    this.dispatchEvent(
      new CustomEvent("update-node", {
        detail: { config },
        bubbles: true,
        composed: true,
      })
    );
  }

  deleteNode() {
    this.dispatchEvent(new CustomEvent("delete-node", { bubbles: true }));
  }

  placeNode(config) {
    this.dispatchEvent(
      new CustomEvent("place-node", { detail: { config }, bubbles: true })
    );
  }

  get width() {
    let w = 0;
    if (this.branching !== undefined) {
      for (const c of this.children) {
        if (c.slot === "head") continue;
        w += (c as any).width;
      }
    } else {
      for (const c of this.children) {
        w = Math.max(w, (c as any).width);
      }
    }
    return w;
  }
  get height() {
    let h = 0;
    if (this.branching !== undefined) {
      for (const c of this.children) {
        if (c.slot === "head") continue;
        h = Math.max(h, (c as any).height);
      }
      h += 2 * BRANCH_HEIGHT;
    } else {
      for (const c of this.children) {
        h += (c as any).height + VERTICAL_SPACING;
      }
      h;
    }
    return h;
  }

  async updateChildren() {
    this._num_items = this.children.length;
  }

  render() {
    let branch_x = [];
    let total = 0;
    if (this.branching !== undefined) {
      for (const c of Array.from(this.children)) {
        if (c.slot === "head") continue;
        const rect = c.getBoundingClientRect();
        branch_x.push(rect.width / 2 + total);
        total += rect.width;
      }
    }

    const line_end = this.height - BRANCH_HEIGHT;

    return html`
      <svg width="${this.width}" height="${this.height}">
        <rect
          x="0"
          y="0"
          width="${this.width}"
          height="${this.height}"
          fill="white"
        />
        ${this.branching !== undefined
          ? branch_x.map((x) => {
              return svg`
                <path
                  class="line"
                  d="
                    M ${this.width / 2} 0
                    C ${this.width / 2} ${BRANCH_CURVATURE}
                      ${x} ${BRANCH_HEIGHT - BRANCH_CURVATURE}
                      ${x} ${BRANCH_HEIGHT}
                    L ${x} ${line_end}
                    C ${x} ${line_end + BRANCH_CURVATURE}
                      ${this.width / 2} ${this.height - BRANCH_CURVATURE}
                      ${this.width / 2} ${this.height}
                  "
                />
              `;
            })
          : svg`
              <path
              class="line"
              d="
                M ${this.width / 2} 0
                L ${this.width / 2} ${this.height}
              "
              />
            `}
      </svg>
      <slot name="head"> </slot>
      <div
        id="branches"
        class="${this.querySelector(":scope > [slot='head']") ? "" : "no-head"}"
      >
        <slot @slotchange=${this.updateChildren}></slot>
      </div>
    `;
  }

  static get styles() {
    return css`
      :host {
        position: relative;
        display: flex;
        --stroke-clr: var(--stroke-color, rgb(3, 169, 244));
        --hover-clr: var(--hover-color, rgb(255, 152, 0));
      }
      #branches {
        position: absolute;
        left: 0;
        display: flex;
        flex-direction: row;
        align-items: center;
      }
      :host([branching]) #branches {
        top: ${BRANCH_HEIGHT}px;
        flex-direction: row;
        align-items: start;
      }
      :host(:not([branching])) #branches {
        top: ${VERTICAL_SPACING + NODE_SIZE}px; /* SHould be something else*/
        flex-direction: column;
      }
      :host(:not([branching])) #branches.no-head {
        top: 0;
      }
      path.line {
        stroke: var(--stroke-clr);
        stroke-width: 2;
        fill: none;
      }
      :host(:not([branching])) ::slotted(*) {
        margin-bottom: 10px;
      }
      ::slotted(:last-child) {
        margin-bottom: 0;
      }
      ::slotted([slot="head"]) {
        position: absolute;
        top: ${BRANCH_HEIGHT / 2}px;
        left: 50%;
        transform: translate(-50%, -50%);
      }
      :host(:focus-within) ::slotted([slot="head"]) {
        --stroke-color: green;
      }
    `;
  }
}

customElements.define("hat-graph", HatGraph);
